#include "log/Log.h"
#include "io/Serial.h"
#include "scheduling/PIT.h"
#include "std/stdio.h"
#include <stdarg.h>

static int level_mask = 15;

void KernelLog::log(const char* function, LogLevel level, const char* message, ...)
{
    if (!(level_mask & (1 << (int)level))) return;
    va_list ap;
    va_start(ap, message);
    Serial::reset_color();
    printf("[%ld.%ld] ", PIT::ms_since_boot / 1000, PIT::ms_since_boot % 1000);
    Serial::print(function);
    Serial::print(": ");
    switch (level)
    {
    case LogLevel::WARN: Serial::set_color(Color::Yellow); break;
    case LogLevel::ERROR: Serial::set_color(Color::Red); break;
    default: break;
    }
    vprintf(message, ap);
    Serial::reset_color();
    va_end(ap);
}

void KernelLog::logln(const char* function, LogLevel level, const char* message, ...)
{
    if (!(level_mask & (1 << (int)level))) return;
    va_list ap;
    va_start(ap, message);
    Serial::reset_color();
    printf("[%ld.%ld] ", PIT::ms_since_boot / 1000, PIT::ms_since_boot % 1000);
    Serial::print(function);
    Serial::print(": ");
    switch (level)
    {
    case LogLevel::WARN: Serial::set_color(Color::Yellow); break;
    case LogLevel::ERROR: Serial::set_color(Color::Red); break;
    default: break;
    }
    vprintf(message, ap);
    Serial::reset_color();
    Serial::print("\n");
    va_end(ap);
}

void KernelLog::toggle_log_level(LogLevel level)
{
    level_mask ^= (1 << (int)level);
}